package com.qq.qcloud.tencent_im_sdk_plugin.manager;

import com.qq.qcloud.tencent_im_sdk_plugin.util.CommonUtil;
import com.tencent.imsdk.v2.V2TIMCallback;
import com.tencent.imsdk.v2.V2TIMFriendAddApplication;
import com.tencent.imsdk.v2.V2TIMFriendApplication;
import com.tencent.imsdk.v2.V2TIMFriendApplicationResult;
import com.tencent.imsdk.v2.V2TIMFriendCheckResult;
import com.tencent.imsdk.v2.V2TIMFriendGroup;
import com.tencent.imsdk.v2.V2TIMFriendInfo;
import com.tencent.imsdk.v2.V2TIMFriendInfoResult;
import com.tencent.imsdk.v2.V2TIMFriendOperationResult;
import com.tencent.imsdk.v2.V2TIMFriendshipListener;
import com.tencent.imsdk.v2.V2TIMManager;
import com.tencent.imsdk.v2.V2TIMValueCallback;

import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;

import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel;

public class FriendshipManager {
    private static MethodChannel channel;
    public FriendshipManager(MethodChannel _channel){
        FriendshipManager.channel = _channel;
    }


    public void setFriendListener(MethodCall methodCall, final MethodChannel.Result result){
        V2TIMManager.getFriendshipManager().setFriendListener(new V2TIMFriendshipListener() {
            @Override
            public void onFriendApplicationListAdded(List<V2TIMFriendApplication> applicationList) {
                HashMap<Object,Object> data = new HashMap<Object,Object>();
                LinkedList<Object> list = new LinkedList<Object>();
                for(int i= 0;i<applicationList.size();i++){
                    list.add(CommonUtil.convertV2TIMFriendApplicationToMap(applicationList.get(i)));
                }
                data.put("applicationList",data);
                makeFriendListenerEventData("onFriendApplicationListAdded",data);
            }

            @Override
            public void onFriendApplicationListDeleted(List<String> userIDList) {
                makeFriendListenerEventData("onFriendApplicationListDeleted",userIDList);
            }

            @Override
            public void onFriendApplicationListRead() {
                makeFriendListenerEventData("onFriendApplicationListRead",null);
            }

            @Override
            public void onFriendListAdded(List<V2TIMFriendInfo> users) {
                HashMap<Object,Object> data = new HashMap<Object,Object>();
                LinkedList<Object> list = new LinkedList<Object>();
                for(int i= 0;i<users.size();i++){
                    list.add(CommonUtil.convertV2TIMFriendInfoToMap(users.get(i)));
                }
                data.put("users",data);
                makeFriendListenerEventData("onFriendListAdded",data);
            }

            @Override
            public void onFriendListDeleted(List<String> userList) {
                makeFriendListenerEventData("onFriendApplicationListDeleted",userList);
            }

            @Override
            public void onBlackListAdd(List<V2TIMFriendInfo> infoList) {
                HashMap<Object,Object> data = new HashMap<Object,Object>();
                LinkedList<Object> list = new LinkedList<Object>();
                for(int i= 0;i<infoList.size();i++){
                    list.add(CommonUtil.convertV2TIMFriendInfoToMap(infoList.get(i)));
                }
                data.put("infoList",data);
                makeFriendListenerEventData("onBlackListAdd",data);
            }

            @Override
            public void onBlackListDeleted(List<String> userList) {
                makeFriendListenerEventData("onBlackListDeleted",userList);
            }

            @Override
            public void onFriendInfoChanged(List<V2TIMFriendInfo> infoList) {
                HashMap<Object,Object> data = new HashMap<Object,Object>();
                LinkedList<Object> list = new LinkedList<Object>();
                for(int i= 0;i<infoList.size();i++){
                    list.add(CommonUtil.convertV2TIMFriendInfoToMap(infoList.get(i)));
                }
                data.put("infoList",data);
                makeFriendListenerEventData("onBlackListAdd",data);
            }
        });
    }
    private <T> void  makeFriendListenerEventData(String type,T data){
        CommonUtil.emitEvent(FriendshipManager.channel,"friendListener",type,data);
    }
    public void getFriendList(MethodCall methodCall, final MethodChannel.Result result){
        V2TIMManager.getFriendshipManager().getFriendList(new V2TIMValueCallback<List<V2TIMFriendInfo>>() {
            @Override
            public void onError(int i, String s) {
                CommonUtil.returnError(result,i,s);
            }

            @Override
            public void onSuccess(List<V2TIMFriendInfo> v2TIMFriendInfos) {
                HashMap<Object,Object> data = new HashMap<Object,Object>();
                LinkedList<Object> list = new LinkedList<Object>();
                for (int i = 0;i<v2TIMFriendInfos.size();i++){
                    list.add(CommonUtil.convertV2TIMFriendInfoToMap(v2TIMFriendInfos.get(0)));
                }
                data.put("v2TIMFriendInfos",list);
                CommonUtil.returnSuccess(result,data);
            }
        });
    }
    public void getFriendsInfo(MethodCall methodCall,final MethodChannel.Result result){
        List<String> userIDList = CommonUtil.getParam(methodCall,result,"userIDList");
        V2TIMManager.getFriendshipManager().getFriendsInfo(userIDList, new V2TIMValueCallback<List<V2TIMFriendInfoResult>>() {
            @Override
            public void onError(int i, String s) {
                CommonUtil.returnError(result,i,s);
            }

            @Override
            public void onSuccess(List<V2TIMFriendInfoResult> v2TIMFriendInfoResults) {
                HashMap<Object,Object> data = new HashMap<Object,Object>();
                LinkedList<Object> list = new LinkedList<Object>();
                for (int i = 0;i<v2TIMFriendInfoResults.size();i++){
                    list.add(CommonUtil.convertV2TIMFriendInfoResultToMap(v2TIMFriendInfoResults.get(0)));
                }
                data.put("v2TIMFriendInfos",list);
                CommonUtil.returnSuccess(result,data);
            }
        });
    }
    public void setFriendInfo(MethodCall methodCall,final MethodChannel.Result result){
        V2TIMManager.getFriendshipManager().setFriendInfo(new V2TIMFriendInfo(), new V2TIMCallback() {
            @Override
            public void onError(int i, String s) {
                CommonUtil.returnError(result,i,s);
            }

            @Override
            public void onSuccess() {
                CommonUtil.returnSuccess(result,null);
            }
        });
    }
    public void addFriend(MethodCall methodCall,final MethodChannel.Result result){
        String userID = CommonUtil.getParam(methodCall,result,"userID");
        V2TIMManager.getFriendshipManager().addFriend(new V2TIMFriendAddApplication(userID), new V2TIMValueCallback<V2TIMFriendOperationResult>() {
            @Override
            public void onError(int i, String s) {
                CommonUtil.returnError(result,i,s);
            }

            @Override
            public void onSuccess(V2TIMFriendOperationResult v2TIMFriendOperationResult) {
                CommonUtil.returnSuccess(result,CommonUtil.convertV2TIMFriendOperationResultToMap(v2TIMFriendOperationResult));
            }
        });
    }
    public void deleteFromFriendList(MethodCall methodCall,final MethodChannel.Result result){
        List<String> userIDList = CommonUtil.getParam(methodCall,result,"userIDList");
        int deleteType = CommonUtil.getParam(methodCall,result,"deleteType");
        V2TIMManager.getFriendshipManager().deleteFromFriendList(userIDList, deleteType, new V2TIMValueCallback<List<V2TIMFriendOperationResult>>() {
            @Override
            public void onError(int i, String s) {
                CommonUtil.returnError(result,i,s);
            }

            @Override
            public void onSuccess(List<V2TIMFriendOperationResult> v2TIMFriendOperationResults) {
                HashMap<Object,Object> data = new HashMap<Object,Object>();
                LinkedList<Object> list = new LinkedList<Object>();
                for (int i = 0;i<v2TIMFriendOperationResults.size();i++){
                    list.add(CommonUtil.convertV2TIMFriendOperationResultToMap(v2TIMFriendOperationResults.get(0)));
                }
                data.put("v2TIMFriendOperationResults",list);
                CommonUtil.returnSuccess(result,data);
            }
        });
    }
    public void checkFriend(MethodCall methodCall,final MethodChannel.Result result){
        String userID = CommonUtil.getParam(methodCall,result,"userID");
        int checkType = CommonUtil.getParam(methodCall,result,"checkType");
        V2TIMManager.getFriendshipManager().checkFriend(userID, checkType, new V2TIMValueCallback<V2TIMFriendCheckResult>() {
            @Override
            public void onError(int i, String s) {
                CommonUtil.returnError(result,i,s);
            }

            @Override
            public void onSuccess(V2TIMFriendCheckResult v2TIMFriendCheckResult) {
                CommonUtil.returnSuccess(result,CommonUtil.convertV2TIMFriendCheckResultToMap(v2TIMFriendCheckResult));
            }
        });
    }
    public void getFriendApplicationList(MethodCall methodCall,final MethodChannel.Result result){
        V2TIMManager.getFriendshipManager().getFriendApplicationList(new V2TIMValueCallback<V2TIMFriendApplicationResult>() {
            @Override
            public void onError(int i, String s) {
                CommonUtil.returnError(result,i,s);
            }

            @Override
            public void onSuccess(V2TIMFriendApplicationResult v2TIMFriendApplicationResult) {
                CommonUtil.returnSuccess(result,CommonUtil.convertV2TIMFriendApplicationResultToMap(v2TIMFriendApplicationResult));
            }
        });
    }
    public void acceptFriendApplication(MethodCall methodCall,final MethodChannel.Result result){
        int responseType = CommonUtil.getParam(methodCall,result,"responseType");
        V2TIMManager.getFriendshipManager().acceptFriendApplication(new V2TIMFriendApplication(), responseType, new V2TIMValueCallback<V2TIMFriendOperationResult>() {
            @Override
            public void onError(int i, String s) {
                CommonUtil.returnError(result,i,s);
            }

            @Override
            public void onSuccess(V2TIMFriendOperationResult v2TIMFriendOperationResult) {
                CommonUtil.returnSuccess(result,CommonUtil.convertV2TIMFriendOperationResultToMap(v2TIMFriendOperationResult));
            }
        });
    }
    public void refuseFriendApplication(MethodCall methodCall,final MethodChannel.Result result){
        V2TIMManager.getFriendshipManager().refuseFriendApplication(new V2TIMFriendApplication(), new V2TIMValueCallback<V2TIMFriendOperationResult>() {
            @Override
            public void onError(int i, String s) {
                CommonUtil.returnError(result,i,s);
            }

            @Override
            public void onSuccess(V2TIMFriendOperationResult v2TIMFriendOperationResult) {
                CommonUtil.returnSuccess(result,CommonUtil.convertV2TIMFriendOperationResultToMap(v2TIMFriendOperationResult));
            }
        });
    }
    public void deleteFriendApplication(MethodCall methodCall,final MethodChannel.Result result){
        V2TIMManager.getFriendshipManager().deleteFriendApplication(new V2TIMFriendApplication(), new V2TIMCallback() {
            @Override
            public void onError(int i, String s) {
                CommonUtil.returnError(result,i,s);
            }

            @Override
            public void onSuccess() {
                CommonUtil.returnSuccess(result,null);
            }
        });
    }
    public void setFriendApplicationRead(MethodCall methodCall,final MethodChannel.Result result){
        V2TIMManager.getFriendshipManager().setFriendApplicationRead(new V2TIMCallback() {
            @Override
            public void onError(int i, String s) {
                CommonUtil.returnError(result,i,s);
            }

            @Override
            public void onSuccess() {
                CommonUtil.returnSuccess(result,null);
            }
        });
    }
    public void addToBlackList(MethodCall methodCall,final MethodChannel.Result result){
        List<String> userIDList = CommonUtil.getParam(methodCall,result,"userIDList");
        V2TIMManager.getFriendshipManager().addToBlackList(userIDList, new V2TIMValueCallback<List<V2TIMFriendOperationResult>>() {
            @Override
            public void onError(int i, String s) {
                CommonUtil.returnError(result,i,s);
            }

            @Override
            public void onSuccess(List<V2TIMFriendOperationResult> v2TIMFriendOperationResults) {
                HashMap<Object,Object> data = new HashMap<Object,Object>();
                LinkedList<Object> list = new LinkedList<Object>();
                for (int i = 0;i<v2TIMFriendOperationResults.size();i++){
                    list.add(CommonUtil.convertV2TIMFriendOperationResultToMap(v2TIMFriendOperationResults.get(0)));
                }
                data.put("v2TIMFriendOperationResults",list);
                CommonUtil.returnSuccess(result,data);
            }
        });
    }
    public void deleteFromBlackList(MethodCall methodCall,final MethodChannel.Result result){
        List<String> userIDList = CommonUtil.getParam(methodCall,result,"userIDList");
        V2TIMManager.getFriendshipManager().deleteFromBlackList(userIDList, new V2TIMValueCallback<List<V2TIMFriendOperationResult>>() {
            @Override
            public void onError(int i, String s) {
                CommonUtil.returnError(result,i,s);
            }

            @Override
            public void onSuccess(List<V2TIMFriendOperationResult> v2TIMFriendOperationResults) {
                HashMap<Object,Object> data = new HashMap<Object,Object>();
                LinkedList<Object> list = new LinkedList<Object>();
                for (int i = 0;i<v2TIMFriendOperationResults.size();i++){
                    list.add(CommonUtil.convertV2TIMFriendOperationResultToMap(v2TIMFriendOperationResults.get(0)));
                }
                data.put("v2TIMFriendOperationResults",list);
                CommonUtil.returnSuccess(result,data);
            }
        });
    }
    public void getBlackList(MethodCall methodCall,final MethodChannel.Result result){
        V2TIMManager.getFriendshipManager().getBlackList(new V2TIMValueCallback<List<V2TIMFriendInfo>>() {
            @Override
            public void onError(int i, String s) {
                CommonUtil.returnError(result,i,s);
            }

            @Override
            public void onSuccess(List<V2TIMFriendInfo> v2TIMFriendInfos) {
                HashMap<Object,Object> data = new HashMap<Object,Object>();
                LinkedList<Object> list = new LinkedList<Object>();
                for (int i = 0;i<v2TIMFriendInfos.size();i++){
                    list.add(CommonUtil.convertV2TIMFriendInfoToMap(v2TIMFriendInfos.get(0)));
                }
                data.put("v2TIMFriendInfos",list);
                CommonUtil.returnSuccess(result,data);
            }
        });
    }
    public void createFriendGroup(MethodCall methodCall,final MethodChannel.Result result){
        String groupName = CommonUtil.getParam(methodCall,result,"groupName");
        List< String > userIDList = CommonUtil.getParam(methodCall,result,"userIDList");
        V2TIMManager.getFriendshipManager().createFriendGroup(groupName, userIDList, new V2TIMValueCallback<List<V2TIMFriendOperationResult>>() {
            @Override
            public void onError(int i, String s) {
                CommonUtil.returnError(result,i,s);
            }

            @Override
            public void onSuccess(List<V2TIMFriendOperationResult> v2TIMFriendOperationResults) {
                HashMap<Object,Object> data = new HashMap<Object,Object>();
                LinkedList<Object> list = new LinkedList<Object>();
                for (int i = 0;i<v2TIMFriendOperationResults.size();i++){
                    list.add(CommonUtil.convertV2TIMFriendOperationResultToMap(v2TIMFriendOperationResults.get(0)));
                }
                data.put("v2TIMFriendOperationResults",list);
                CommonUtil.returnSuccess(result,data);
            }
        });
    }
    public void getFriendGroups(MethodCall methodCall,final MethodChannel.Result result){
        List< String > groupNameList = CommonUtil.getParam(methodCall,result,"groupNameList");
        V2TIMManager.getFriendshipManager().getFriendGroups(groupNameList, new V2TIMValueCallback<List<V2TIMFriendGroup>>() {
            @Override
            public void onError(int i, String s) {
                CommonUtil.returnError(result,i,s);
            }

            @Override
            public void onSuccess(List<V2TIMFriendGroup> v2TIMFriendGroups) {
                HashMap<Object,Object> data = new HashMap<Object,Object>();
                LinkedList<Object> list = new LinkedList<Object>();
                for (int i = 0;i<v2TIMFriendGroups.size();i++){
                    list.add(CommonUtil.convertV2TIMFriendGroupToMap(v2TIMFriendGroups.get(0)));
                }
                data.put("v2TIMFriendGroups",list);
                CommonUtil.returnSuccess(result,data);
            }
        });
    }
    public void deleteFriendGroup(MethodCall methodCall,final MethodChannel.Result result){
        List< String > groupNameList = CommonUtil.getParam(methodCall,result,"groupNameList");
        V2TIMManager.getFriendshipManager().deleteFriendGroup(groupNameList, new V2TIMCallback() {
            @Override
            public void onError(int i, String s) {
                CommonUtil.returnError(result,i,s);
            }

            @Override
            public void onSuccess() {
                CommonUtil.returnSuccess(result,null);
            }
        });
    }
    public void renameFriendGroup(MethodCall methodCall,final MethodChannel.Result result){
        String oldName = CommonUtil.getParam(methodCall,result,"oldName");
        String newName = CommonUtil.getParam(methodCall,result,"newName");
        V2TIMManager.getFriendshipManager().renameFriendGroup(oldName, newName, new V2TIMCallback() {
            @Override
            public void onError(int i, String s) {
                CommonUtil.returnError(result,i,s);
            }

            @Override
            public void onSuccess() {
                CommonUtil.returnSuccess(result,null);
            }
        });
    }
    public void addFriendsToFriendGroup(MethodCall methodCall,final MethodChannel.Result result){
        String groupName = CommonUtil.getParam(methodCall,result,"groupName");
        List< String > userIDList = CommonUtil.getParam(methodCall,result,"userIDList");
        V2TIMManager.getFriendshipManager().addFriendsToFriendGroup(groupName, userIDList, new V2TIMValueCallback<List<V2TIMFriendOperationResult>>() {
            @Override
            public void onError(int i, String s) {
                CommonUtil.returnError(result,i,s);
            }

            @Override
            public void onSuccess(List<V2TIMFriendOperationResult> v2TIMFriendOperationResults) {
                HashMap<Object,Object> data = new HashMap<Object,Object>();
                LinkedList<Object> list = new LinkedList<Object>();
                for (int i = 0;i<v2TIMFriendOperationResults.size();i++){
                    list.add(CommonUtil.convertV2TIMFriendOperationResultToMap(v2TIMFriendOperationResults.get(0)));
                }
                data.put("v2TIMFriendOperationResults",list);
                CommonUtil.returnSuccess(result,data);
            }
        });
    }
    public void deleteFriendsFromFriendGroup(MethodCall methodCall,final MethodChannel.Result result){
        String groupName =  CommonUtil.getParam(methodCall,result,"groupName");
        List< String > userIDList  = CommonUtil.getParam(methodCall,result,"userIDList");
        V2TIMManager.getFriendshipManager().deleteFriendsFromFriendGroup(groupName, userIDList, new V2TIMValueCallback<List<V2TIMFriendOperationResult>>() {
            @Override
            public void onError(int i, String s) {
                CommonUtil.returnError(result,i,s);
            }

            @Override
            public void onSuccess(List<V2TIMFriendOperationResult> v2TIMFriendOperationResults) {
                HashMap<Object,Object> data = new HashMap<Object,Object>();
                LinkedList<Object> list = new LinkedList<Object>();
                for (int i = 0;i<v2TIMFriendOperationResults.size();i++){
                    list.add(CommonUtil.convertV2TIMFriendOperationResultToMap(v2TIMFriendOperationResults.get(0)));
                }
                data.put("v2TIMFriendOperationResults",list);
                CommonUtil.returnSuccess(result,data);
            }
        });
    }

}
