import 'package:tencent_im_sdk_plugin/class/V2TIMFriendAddApplication.dart';
import 'package:tencent_im_sdk_plugin/class/V2TIMFriendApplication.dart';
import 'package:tencent_im_sdk_plugin/class/V2TIMFriendInfo.dart';
import 'package:flutter/services.dart';

class V2TIMFriendshipManager {
  MethodChannel _channel = null;
  Set friendListener = new Set();
  V2TIMFriendshipManager(channel) {
    _channel = channel;
  }
  void setFriendListener(Function listener) {
    this.friendListener.add(listener);
    _channel.invokeMethod("setFriendListener", buildParam({}));
  }

  Future<Map> getFriendList() async {
    return await _channel.invokeMethod("getFriendList", buildParam({}));
  }

  Future<Map> getFriendsInfo(
    List<String> userIDList,
  ) async {
    return await _channel.invokeMethod(
        "getFriendsInfo", buildParam({"userIDList": userIDList}));
  }

  Future<Map> setFriendInfo(
    V2TIMFriendInfo info,
  ) async {
    return await _channel.invokeMethod(
        "setFriendInfo",
        buildParam({
          "info": new Map(),
        }));
  }

  Future<Map> addFriend(
    V2TIMFriendAddApplication application,
  ) async {
    return await _channel.invokeMethod(
        "addFriend",
        buildParam({
          "application": new Map(),
        }));
  }

  Future<Map> deleteFromFriendList(
    List<String> userIDList,
    int deleteType,
  ) async {
    return await _channel.invokeMethod(
        "deleteFromFriendList",
        buildParam({
          "userIDList": userIDList,
          "deleteType": deleteType,
        }));
  }

  Future<Map> checkFriend(
    String userID,
    int checkType,
  ) async {
    return await _channel.invokeMethod(
        "checkFriend",
        buildParam({
          "userID": userID,
          "checkType": checkType,
        }));
  }

  Future<Map> getFriendApplicationList() async {
    return await _channel.invokeMethod(
        "getFriendApplicationList", buildParam({}));
  }

  Future<Map> acceptFriendApplication(
    V2TIMFriendApplication application,
    int responseType,
  ) async {
    return await _channel.invokeMethod(
        "acceptFriendApplication",
        buildParam({
          "responseType": responseType,
        }));
  }

  Future<Map> refuseFriendApplication(
    V2TIMFriendApplication application,
  ) async {
    return await _channel.invokeMethod(
        "refuseFriendApplication",
        buildParam({
          "application": application,
        }));
  }

  Future<Map> deleteFriendApplication(
    V2TIMFriendApplication application,
  ) async {
    return await _channel.invokeMethod(
        "deleteFriendApplication",
        buildParam({
          "application": application,
        }));
  }

  Future<Map> setFriendApplicationRead() async {
    return await _channel.invokeMethod(
        "setFriendApplicationRead", buildParam({}));
  }

  Future<Map> addToBlackList(
    List<String> userIDList,
  ) async {
    return await _channel.invokeMethod(
        "addToBlackList",
        buildParam({
          "userIDList": userIDList,
        }));
  }

  Future<Map> deleteFromBlackList(
    List<String> userIDList,
  ) async {
    return await _channel.invokeMethod(
        "deleteFromBlackList",
        buildParam({
          "userIDList": userIDList,
        }));
  }

  Future<Map> getBlackList() async {
    return await _channel.invokeMethod("getBlackList", buildParam({}));
  }

  Future<Map> createFriendGroup(
    String groupName,
    List<String> userIDList,
  ) async {
    return await _channel.invokeMethod(
        "createFriendGroup",
        buildParam({
          "groupName": groupName,
          "userIDList": userIDList,
        }));
  }

  Future<Map> getFriendGroups(
    List<String> groupNameList,
  ) async {
    return await _channel.invokeMethod(
        "getFriendGroups",
        buildParam({
          "groupNameList": groupNameList,
        }));
  }

  Future<Map> deleteFriendGroup(
    List<String> groupNameList,
  ) async {
    return await _channel.invokeMethod(
        "deleteFriendGroup",
        buildParam({
          "groupNameList": groupNameList,
        }));
  }

  Future<Map> renameFriendGroup(
    String oldName,
    String newName,
  ) async {
    return await _channel.invokeMethod(
        "renameFriendGroup",
        buildParam({
          "oldName": oldName,
          "newName": newName,
        }));
  }

  Future<Map> addFriendsToFriendGroup(
    String groupName,
    List<String> userIDList,
  ) async {
    return await _channel.invokeMethod(
        "addFriendsToFriendGroup",
        buildParam({
          "groupName": groupName,
          "userIDList": userIDList,
        }));
  }

  Future<Map> deleteFriendsFromFriendGroup(
    String groupName,
    List<String> userIDList,
  ) async {
    return await _channel.invokeMethod(
        "deleteFriendsFromFriendGroup",
        buildParam({
          "groupName": groupName,
          "userIDList": userIDList,
        }));
  }

  Map buildParam(Map param) {
    param["TIMManagerName"] = "friendshipManager";
    return param;
  }
}
