import 'package:tencent_im_sdk_plugin/class/V2TIMCreateGroupMemberInfo.dart';
import 'package:tencent_im_sdk_plugin/class/V2TIMGroupApplication.dart';
import 'package:tencent_im_sdk_plugin/class/V2TIMGroupInfo.dart';
import 'package:tencent_im_sdk_plugin/class/V2TIMGroupMemberFullInfo.dart';
import 'package:flutter/services.dart';

class V2TIMGroupManager {
  MethodChannel _channel = null;
  V2TIMGroupManager(channel) {
    _channel = channel;
  }
  Future<void> createGroup(
      V2TIMGroupInfo info, List<V2TIMCreateGroupMemberInfo> memberList) async {
    await _channel.invokeMethod(
        "createGroup",
        buildParam({
          "info": info,
          "memberList": memberList,
        }));
  }

  Future<Map> getJoinedGroupList() async {
    return await _channel.invokeMethod("getJoinedGroupList", buildParam({}));
  }

  Future<Map> getGroupsInfo(List<String> groupIDList) async {
    return await _channel.invokeMethod(
        "getGroupsInfo",
        buildParam({
          "groupIDList": groupIDList,
        }));
  }

  Future<Map> setGroupInfo(V2TIMGroupInfo info) async {
    return await _channel.invokeMethod(
        "setGroupInfo",
        buildParam({
          "info": info,
        }));
  }

  Future<Map> setReceiveMessageOpt(String groupID, int opt) async {
    return await _channel.invokeMethod(
        "setReceiveMessageOpt",
        buildParam({
          "groupID": groupID,
          "opt": opt,
        }));
  }

  Future<Map> initGroupAttributes(
      String groupID, Map<String, String> attributes) async {
    return await _channel.invokeMethod(
        "initGroupAttributes",
        buildParam({
          "groupID": groupID,
          "attributes": attributes,
        }));
  }

  Future<Map> setGroupAttributes(
      String groupID, Map<String, String> attributes) async {
    return await _channel.invokeMethod(
        "setGroupAttributes",
        buildParam({
          "groupID": groupID,
          "attributes": attributes,
        }));
  }

  Future<Map> deleteGroupAttributes(
    String groupID,
    List<String> keys,
  ) async {
    return await _channel.invokeMethod(
        "deleteGroupAttributes",
        buildParam({
          "groupID": groupID,
          "keys": keys,
        }));
  }

  Future<Map> getGroupAttributes(
    String groupID,
    List<String> keys,
  ) async {
    return await _channel.invokeMethod(
        "getGroupAttributes",
        buildParam({
          "groupID": groupID,
          "keys": keys,
        }));
  }

  Future<Map> getGroupMemberList(
    String groupID,
    int filter,
    double nextSeq,
  ) async {
    return await _channel.invokeMethod(
        "getGroupMemberList",
        buildParam({
          "groupID": groupID,
          "filter": filter,
          "nextSeq": nextSeq,
        }));
  }

  Future<Map> getGroupMembersInfo(
    String groupID,
    List<String> memberList,
  ) async {
    return await _channel.invokeMethod(
        "getGroupMembersInfo",
        buildParam({
          "groupID": groupID,
          "memberList": memberList,
        }));
  }

  Future<Map> setGroupMemberInfo(
    String groupID,
    V2TIMGroupMemberFullInfo info,
  ) async {
    return await _channel.invokeMethod(
        "setGroupMemberInfo",
        buildParam({
          "info": info,
          "groupID": groupID,
        }));
  }

  Future<Map> muteGroupMember(
    String groupID,
    String userID,
    int seconds,
  ) async {
    return await _channel.invokeMethod(
        "muteGroupMember",
        buildParam({
          "groupID": groupID,
          "userID": userID,
          "seconds": seconds,
        }));
  }

  Future<Map> inviteUserToGroup(
    String groupID,
    List<String> userList,
  ) async {
    return await _channel.invokeMethod(
        "inviteUserToGroup",
        buildParam({
          "groupID": groupID,
          "userList": userList,
        }));
  }

  Future<Map> kickGroupMember(
    String groupID,
    List<String> memberList,
    String reason,
  ) async {
    return await _channel.invokeMethod(
        "kickGroupMember",
        buildParam({
          "groupID": groupID,
          "memberList": memberList,
          "reason": reason,
        }));
  }

  Future<Map> setGroupMemberRole(
    String groupID,
    String userID,
    int role,
  ) async {
    return await _channel.invokeMethod(
        "setGroupMemberRole",
        buildParam({
          "groupID": groupID,
          "userID": userID,
          "role": role,
        }));
  }

  Future<Map> transferGroupOwner(
    String groupID,
    String userID,
  ) async {
    return await _channel.invokeMethod(
        "transferGroupOwner",
        buildParam({
          "groupID": groupID,
          "userID": userID,
        }));
  }

  Future<Map> getGroupApplicationList() async {
    return await _channel.invokeMethod(
        "getGroupApplicationList", buildParam({}));
  }

  Future<Map> acceptGroupApplication(
    V2TIMGroupApplication application,
  ) async {
    return await _channel.invokeMethod(
        "acceptGroupApplication",
        buildParam({
          "application": application,
        }));
  }

  Future<Map> refuseGroupApplication(
    V2TIMGroupApplication application,
    String reason,
  ) async {
    return await _channel.invokeMethod(
        "refuseGroupApplication",
        buildParam({
          "application": application,
        }));
  }

  Future<Map> setGroupApplicationRead() async {
    return await _channel.invokeMethod(
        "setGroupApplicationRead", buildParam({}));
  }

  Map buildParam(Map param) {
    param["TIMManagerName"] = "groupManager";
    return param;
  }
}
