import 'package:tencent_im_sdk_plugin/class/V2TIMCallback.dart';
import 'package:tencent_im_sdk_plugin/manager/V2TIMConversationManager.dart';
import 'package:tencent_im_sdk_plugin/manager/V2TIMFriendshipManager.dart';
import 'package:tencent_im_sdk_plugin/manager/V2TIMGroupManager.dart';
import 'package:tencent_im_sdk_plugin/manager/V2TIMMessageManager.dart';
import 'package:tencent_im_sdk_plugin/manager/V2TIMOfflinePushManager.dart';
import 'package:tencent_im_sdk_plugin/manager/V2TIMSignalingManager.dart';
import 'package:flutter/services.dart';

class V2TIMManager {
  MethodChannel _channel = null;
  V2TIMConversationManager v2ConversationManager = null;
  V2TIMMessageManager v2TIMMessageManager = null;
  V2TIMFriendshipManager v2TIMFriendshipManager = null;
  V2TIMGroupManager v2TIMGroupManager = null;
  V2TIMOfflinePushManager v2TIMOfflinePushManager = null;
  V2TIMSignalingManager v2timSignalingManager = null;
  Set simpleMsgListener = new Set();
  Set initSDKListener = new Set();
  Set groupListener = new Set();
  V2TIMManager(MethodChannel channel) {
    this._channel = channel;
    this.v2ConversationManager = new V2TIMConversationManager(channel);
    this.v2TIMMessageManager = new V2TIMMessageManager(channel);
    this.v2TIMFriendshipManager = new V2TIMFriendshipManager(channel);
    this.v2TIMGroupManager = new V2TIMGroupManager(channel);
    this.v2TIMOfflinePushManager = new V2TIMOfflinePushManager(channel);
    this.v2timSignalingManager = new V2TIMSignalingManager(channel);
    this.addNativeCallback(channel);
  }

  void addNativeCallback(_channel) {
    _channel.setMethodCallHandler((call) {
      if (call.method == "test") {
        return null;
      } else if (call.method == "simpleMsgListener") {
        for (var item in simpleMsgListener) {
          item(call.arguments);
        }
      } else if (call.method == "initSDKListener") {
        for (var item in initSDKListener) {
          item(call.arguments);
        }
      } else if (call.method == "groupListener") {
        for (var item in groupListener) {
          item(call.arguments);
        }
      } else if (call.method == "advancedMsgListener") {
        for (var item in this.v2TIMMessageManager.advancedMsgListener) {
          item(call.arguments);
        }
      } else {
        return null;
      }
    });
  }

  Future<bool> initSDK(int sdkAppID, int loglevel, Function listener) async {
    initSDKListener.add(listener);
    return await _channel.invokeMethod(
        'initSDK',
        buildParam({
          "sdkAppID": sdkAppID,
          "logLevel": loglevel,
        }));
  }

  Future<bool> unInitSDK() async {
    initSDKListener.clear();
    await _channel.invokeMethod('unInitSDK', buildParam({}));
  }

  Future<Map> login(String userID, String userSig) async {
    return await _channel.invokeMethod(
        'login',
        buildParam({
          "userID": userID,
          "userSig": userSig,
        }));
  }

  Future<Map> logout(V2TIMCallback callback) async {
    return await _channel.invokeMethod('logout', buildParam({}));
  }

  Future<String> getLoginUser() async {
    return await _channel.invokeMethod('getLoginUser', buildParam({}));
  }

  Future<int> getLoginStatus() async {
    return await _channel.invokeMethod('getLoginStatus', buildParam({}));
  }

  void addSimpleMsgListener(Function simpleMsgListener) {
    this.simpleMsgListener.add(simpleMsgListener);
    _channel.invokeMethod("addSimpleMsgListener", buildParam({}));
  }

  void removeSimpleMsgListener() {
    this.simpleMsgListener.clear();
    _channel.invokeMethod("removeSimpleMsgListener", buildParam({}));
  }

  Future<Map> sendC2CTextMessage(
    String text,
    String userID,
  ) async {
    return await _channel.invokeMethod(
        'sendC2CTextMessage',
        buildParam({
          "text": text,
          "userID": userID,
        }));
  }

  Future<Map> sendC2CCustomMessage(
    String customData,
    String userID,
  ) async {
    return await _channel.invokeMethod(
        'sendC2CCustomMessage',
        buildParam({
          "customData": customData,
          "userID": userID,
        }));
  }

  Future<Map> sendGroupTextMessage(
    String text,
    String groupID,
    int priority,
  ) async {
    return await _channel.invokeMethod(
        "sendGroupTextMessage",
        buildParam({
          "text": text,
          "groupID": groupID,
          "priority": priority,
        }));
  }

  Future<Map> sendGroupCustomMessage(
      String customData, String groupID, int priority) async {
    return await _channel.invokeMethod(
        "sendGroupCustomMessage",
        buildParam({
          "customData": customData,
          "groupID": groupID,
          "priority": priority,
        }));
  }

  void setGroupListener(Function listener) {
    groupListener.add(listener);
    _channel.invokeMethod("setGroupListener", buildParam({}));
  }

  Future<Map> createGroup(String groupType, String groupName,
      [String groupID]) async {
    return await _channel.invokeMethod(
        "createGroup",
        buildParam({
          "groupType": groupType,
          "groupName": groupName,
          "groupID": groupID,
        }));
  }

  Future<Map> joinGroup(
    String groupID,
    String message,
  ) async {
    return await _channel.invokeMethod(
        "joinGroup",
        buildParam({
          "groupID": groupID,
          "message": message,
        }));
  }

  Future<Map> quitGroup(
    String groupID,
  ) async {
    return await _channel.invokeMethod(
        "quitGroup",
        buildParam({
          "groupID": groupID,
        }));
  }

  Future<Map> dismissGroup(String groupID) async {
    return await _channel.invokeMethod(
        "dismissGroup",
        buildParam({
          "groupID": groupID,
        }));
  }

  Future<Map> getUsersInfo(List<String> userIDList) async {
    return await _channel.invokeMethod(
        "getUsersInfo",
        buildParam({
          "userIDList": userIDList,
        }));
  }

  Future<Map> setSelfInfo({
    String nickName,
    String faceUrl,
    String selfSignature,
    int gender,
    int allowType,
  }) async {
    return await _channel.invokeMethod(
        "setSelfInfo",
        buildParam({
          "nickName": nickName,
          "faceUrl": faceUrl,
          "selfSignature": selfSignature,
          "gender": gender,
          "allowType": allowType,
        }));
  }

  V2TIMMessageManager getMessageManager() {
    return this.v2TIMMessageManager;
  }

  V2TIMGroupManager getGroupManager() {
    return this.v2TIMGroupManager;
  }

  V2TIMConversationManager getConversationManager() {
    return this.v2ConversationManager;
  }

  Map buildParam(Map param) {
    param["TIMManagerName"] = "timManager";
    return param;
  }

  V2TIMFriendshipManager getFriendshipManager() {
    return this.v2TIMFriendshipManager;
  }

  V2TIMOfflinePushManager getOfflinePushManager() {
    return this.v2TIMOfflinePushManager;
  }

  V2TIMSignalingManager getSignalingManager() {
    return this.v2timSignalingManager;
  }

  static final int V2TIM_STATUS_LOGINED = 1;

  static final int V2TIM_STATUS_LOGINING = 2;

  static final int V2TIM_STATUS_LOGOUT = 3;

  static final String GROUP_TYPE_WORK = "Work";

  static final String GROUP_TYPE_PUBLIC = "Public";

  static final String GROUP_TYPE_MEETING = "Meeting";

  static final String GROUP_TYPE_AVCHATROOM = "AVChatRoom";
}
