import 'package:tencent_im_sdk_plugin/class/V2TIMMessage.dart';
import 'package:tencent_im_sdk_plugin/class/V2TIMOfflinePushInfo.dart';

import 'package:flutter/services.dart';

class V2TIMMessageManager {
  MethodChannel _channel = null;
  Set advancedMsgListener = new Set();
  V2TIMMessageManager(channel) {
    this._channel = channel;
  }
  void addAdvancedMsgListener(Function listener) {
    advancedMsgListener.add(listener);
    _channel.invokeMethod("addAdvancedMsgListener", buildParam({}));
  }

  void removeAdvancedMsgListener(Function listener) {
    advancedMsgListener.clear();
    _channel.invokeMethod("removeAdvancedMsgListener", buildParam({}));
  }

  Future<Map> createTextMessage(String text) async {
    return await _channel.invokeMethod(
        "createTextMessage",
        buildParam({
          "text": text,
        }));
  }

  Future<void> createCustomMessage(List<String> data) async {
    await _channel.invokeMethod(
        "createCustomMessage",
        buildParam({
          "data": data,
        }));
  }

  Future<void> createImageMessage(String imagePath) async {
    await _channel.invokeMethod(
        "createImageMessage",
        buildParam({
          "imagePath": imagePath,
        }));
  }

  Future<void> createSoundMessage(String soundPath, int duration) async {
    await _channel.invokeMethod(
        "createSoundMessage",
        buildParam({
          "soundPath": soundPath,
          "duration": duration,
        }));
  }

  Future<void> createVideoMessage(String videoFilePath, String type,
      int duration, String snapshotPath) async {
    await _channel.invokeMethod(
        "createVideoMessage",
        buildParam({
          "videoFilePath": videoFilePath,
          "type": type,
        }));
  }

  Future<void> createFileMessage(String filePath, String fileName) async {
    await _channel.invokeMethod(
        "createFileMessage",
        buildParam({
          "filePath": filePath,
          "fileName": fileName,
        }));
  }

  Future<void> createLocationMessage(
      String desc, double longitude, double latitude) async {
    await _channel.invokeMethod(
        "createLocationMessage",
        buildParam({
          "desc": desc,
          "longitude": longitude,
          "latitude": latitude,
        }));
  }

  Future<void> createFaceMessage(int index, List<String> data) async {
    await _channel.invokeMethod(
        "createFaceMessage",
        buildParam({
          "index": index,
          "data": data,
        }));
  }

  Future<Map> sendTextMessage(
    String text,
    String receiver,
    String groupID,
    int priority,
    bool onlineUserOnly,
    V2TIMOfflinePushInfo offlinePushInfo,
  ) async {
    return await _channel.invokeMethod(
        "sendMessage",
        buildParam({
          "text": text,
          "receiver": receiver,
          "groupID": groupID,
          "priority": priority,
          "onlineUserOnly": onlineUserOnly,
          "offlinePushInfo": offlinePushInfo
        }));
  }

  Future<Map> sendCustomMessage(
    String data,
    String receiver,
    String groupID,
    int priority,
    bool onlineUserOnly,
    V2TIMOfflinePushInfo offlinePushInfo,
  ) async {
    return await _channel.invokeMethod(
        "sendMessage",
        buildParam({
          "data": data,
          "receiver": receiver,
          "groupID": groupID,
          "priority": priority,
          "onlineUserOnly": onlineUserOnly,
          "offlinePushInfo": offlinePushInfo
        }));
  }

  Future<Map> sendImageMessage(
    String imagePath,
    String receiver,
    String groupID,
    int priority,
    bool onlineUserOnly,
    V2TIMOfflinePushInfo offlinePushInfo,
  ) async {
    return await _channel.invokeMethod(
        "sendMessage",
        buildParam({
          "imagePath": imagePath,
          "receiver": receiver,
          "groupID": groupID,
          "priority": priority,
          "onlineUserOnly": onlineUserOnly,
          "offlinePushInfo": offlinePushInfo
        }));
  }

  Future<Map> sendSoundMessage(
    String soundPath,
    String receiver,
    String groupID,
    int priority,
    bool onlineUserOnly,
    V2TIMOfflinePushInfo offlinePushInfo,
  ) async {
    return await _channel.invokeMethod(
        "sendMessage",
        buildParam({
          "soundPath": soundPath,
          "receiver": receiver,
          "groupID": groupID,
          "priority": priority,
          "onlineUserOnly": onlineUserOnly,
          "offlinePushInfo": offlinePushInfo
        }));
  }

  Future<Map> sendVideoMessage(
    String videoFilePath,
    String receiver,
    String groupID,
    int priority,
    bool onlineUserOnly,
    V2TIMOfflinePushInfo offlinePushInfo,
  ) async {
    return await _channel.invokeMethod(
        "sendMessage",
        buildParam({
          "videoFilePath": videoFilePath,
          "receiver": receiver,
          "groupID": groupID,
          "priority": priority,
          "onlineUserOnly": onlineUserOnly,
          "offlinePushInfo": offlinePushInfo
        }));
  }

  Future<Map> sendFileMessage(
    String filePath,
    String fileName,
    String receiver,
    String groupID,
    int priority,
    bool onlineUserOnly,
    V2TIMOfflinePushInfo offlinePushInfo,
  ) async {
    return await _channel.invokeMethod(
        "sendMessage",
        buildParam({
          "filePath": filePath,
          "fileName": fileName,
          "receiver": receiver,
          "groupID": groupID,
          "priority": priority,
          "onlineUserOnly": onlineUserOnly,
          "offlinePushInfo": offlinePushInfo
        }));
  }

  Future<Map> sendLocationMessage(
    String desc,
    double longitude,
    double latitude,
    String receiver,
    String groupID,
    int priority,
    bool onlineUserOnly,
    V2TIMOfflinePushInfo offlinePushInfo,
  ) async {
    return await _channel.invokeMethod(
        "sendMessage",
        buildParam({
          "desc": desc,
          "longitude": longitude,
          "latitude": latitude,
          "receiver": receiver,
          "groupID": groupID,
          "priority": priority,
          "onlineUserOnly": onlineUserOnly,
          "offlinePushInfo": offlinePushInfo
        }));
  }

  Future<Map> sendFaceMessage(
    String desc,
    String data,
    String receiver,
    String groupID,
    int priority,
    bool onlineUserOnly,
    V2TIMOfflinePushInfo offlinePushInfo,
  ) async {
    return await _channel.invokeMethod(
        "sendMessage",
        buildParam({
          "desc": desc,
          "data": data,
          "receiver": receiver,
          "groupID": groupID,
          "priority": priority,
          "onlineUserOnly": onlineUserOnly,
          "offlinePushInfo": offlinePushInfo
        }));
  }

  Future<Map> getC2CHistoryMessageList(
    String userID,
    int count,
    V2TIMMessage lastMsg,
  ) async {
    return await _channel.invokeMethod(
        "getC2CHistoryMessageList",
        buildParam({
          "userID": userID,
          "count": count,
          "lastMsg": lastMsg,
        }));
  }

  Future<Map> getGroupHistoryMessageList(
    String groupID,
    int count,
    V2TIMMessage lastMsg,
  ) async {
    return await _channel.invokeMethod(
        "getGroupHistoryMessageList",
        buildParam({
          "groupID": groupID,
          "count": count,
          "lastMsg": lastMsg,
        }));
  }

  Future<void> revokeMessage(
    V2TIMMessage msg,
  ) async {
    await _channel.invokeMethod(
        "revokeMessage",
        buildParam({
          "msg": msg,
        }));
  }

  Future<Map> markC2CMessageAsRead(
    String userID,
  ) async {
    return await _channel.invokeMethod(
        "markC2CMessageAsRead",
        buildParam({
          "userID": userID,
        }));
  }

  Future<Map> markGroupMessageAsRead(
    String groupID,
  ) async {
    return await _channel.invokeMethod(
        "markGroupMessageAsRead",
        buildParam({
          "groupID": groupID,
        }));
  }

  Future<Map> deleteMessageFromLocalStorage(
    V2TIMMessage msg,
  ) async {
    return await _channel.invokeMethod(
        "deleteMessageFromLocalStorage",
        buildParam({
          "msg": msg,
        }));
  }

  Future<Map> insertGroupMessageToLocalStorage(
    V2TIMMessage msg,
    String groupID,
    String sender,
  ) async {
    return await _channel.invokeMethod(
        "insertGroupMessageToLocalStorage",
        buildParam({
          "msg": msg,
          "groupID": groupID,
          "sender": sender,
        }));
  }

  Map buildParam(Map param) {
    param["TIMManagerName"] = "messageManager";
    return param;
  }
}
