import 'package:tencent_im_sdk_plugin/class/V2TIMMessage.dart';
import 'package:tencent_im_sdk_plugin/class/V2TIMSignalingInfo.dart';
import 'package:flutter/services.dart';

class V2TIMSignalingManager {
  MethodChannel _channel = null;
  V2TIMSignalingManager(channel) {
    _channel = channel;
  }
  void addSignalingListener(Function listener) {
    _channel.invokeMethod("addSignalingListener", buildParam({}));
  }

  void removeSignalingListener(Function listener) {
    _channel.invokeMethod("removeSignalingListener", buildParam({}));
  }

  Future<Map> invite(String invitee, String data, int timeout) async {
    return await _channel.invokeMethod(
        "invite",
        buildParam({
          "invitee": invitee,
          "data": data,
          "timeout": timeout,
        }));
  }

  Future<Map> inviteInGroup(String groupID, List<String> inviteeList,
      String data, int timeout) async {
    return await _channel.invokeMethod(
        "inviteInGroup",
        buildParam({
          "groupID": groupID,
          "inviteeList": inviteeList,
          "data": data,
          "timeout": timeout,
        }));
  }

  Future<Map> cancel(String inviteID, String data) async {
    return await _channel.invokeMethod(
        "cancel",
        buildParam({
          "inviteID": inviteID,
          "data": data,
        }));
  }

  Future<Map> accept(String inviteID, String data) async {
    return await _channel.invokeMethod(
        "accept",
        buildParam({
          "inviteID": inviteID,
          "data": data,
        }));
  }

  Future<Map> reject(String inviteID, String data) async {
    return await _channel.invokeMethod(
        "reject",
        buildParam({
          "inviteID": inviteID,
          "data": data,
        }));
  }

  //TODO
  Future<Map> getSignalingInfo(V2TIMMessage msg) async {
    return await _channel.invokeMethod(
        "getSignalingInfo",
        buildParam({
          "msg": msg,
        }));
  }

  Future<Map> addInvitedSignaling(
    String inviteID,
    String groupID,
    List<String> inviteeList,
    String data,
    int timeout,
    int actionType,
    int businessID,
  ) async {
    return await _channel.invokeMethod(
        "addInvitedSignaling",
        buildParam({
          "inviteID": inviteID,
          "groupID": groupID,
          "inviteeList": inviteeList,
          "data": data,
          "timeout": timeout,
          "actionType": actionType,
          "businessID": businessID
        }));
  }

  Map buildParam(Map param) {
    param["TIMManagerName"] = "signalingManager";
    return param;
  }
}
